function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
import ModelListener from './utils/ModelListener';

/**
 * A generic component wrapper that creates components for each key/value pair on a model.
 */
var ModelFragment = /*#__PURE__*/function () {
  /**
   * Creates a new ModelFragment instance
   * @param {Model} [model] Optional model object
   * @param {function} factory Component factory with the signature: (key, value) => Component
   * @param {object} [opt] Optional parameters
   * @param {function} [opt.onAdd] Callback called to render the component on add. Can be used for animation. Defaults to (c, el) => c.render(el)
   * @param {function} [opt.onRemove] Callback called to eventually unrender the component on remove. Defaults to (c, unrender) => unrender()
   */
  function ModelFragment(model, factory, opt) {
    _classCallCheck(this, ModelFragment);
    if (typeof model === 'function') {
      opt = factory;
      factory = model;
      model = null;
    }
    this.opt = opt || {};
    this._factory = factory;
    this._comps = null;
    this._unrendering = null;
    this._unrender = {};
    this._el = null;
    this.ml = new ModelListener(model, this, this._update.bind(this));
  }

  /**
   * Set model.
   * @param {?Model} model Model
   * @returns {this}
   */
  _createClass(ModelFragment, [{
    key: "setModel",
    value: function setModel(model) {
      this.ml.setModel(model);
      return this;
    }

    /**
     * Returns the model.
     * @returns {?Model} Model
     */
  }, {
    key: "getModel",
    value: function getModel() {
      return this.ml.model;
    }
  }, {
    key: "render",
    value: function render(el) {
      this._comps = {};
      this._unrendering = {};
      this._el = el;
      this.ml.onRender();
    }
  }, {
    key: "unrender",
    value: function unrender() {
      if (this._el) {
        for (var k in this._unrendering) {
          var cont = this._unrendering[k];
          if (cont.c) {
            cont.c.unrender();
          }
        }
        for (var _k in this._comps) {
          var _cont = this._comps[_k];
          if (_cont.c) {
            _cont.c.unrender();
          }
        }
        this.ml.onUnrender();
      }
      this._unrendering = null;
      this._comps = null;
      this._el = null;
    }
  }, {
    key: "_update",
    value: function _update(m) {
      var _this = this;
      if (!this._el) return;
      var p = m && _typeof(m) === 'object' && _typeof(m.props) == 'object' ? m.props : m;
      var onAdd = this.ml.rendered && this.opt.onAdd || function (c, el) {
        return c.render(el);
      };
      var onRemove = this.ml.rendered && this.opt.onRemove || function (c, unrender) {
        return unrender();
      };

      // Render components
      if (p) {
        var _loop = function _loop(k) {
          if (!p.hasOwnProperty(k)) {
            return 1; // continue
          }
          var v = p[k];
          var cont = _this._comps[k];

          // Unrender previous component on value change
          if (cont && cont.v !== v) {
            if (cont.c) {
              var oldCont = cont;
              _this._unrendering[k] = oldCont;
              var promise = onRemove(oldCont.c, function () {
                return _this._unrenderCont(k, oldCont);
              });
              // Legacy support for promises.
              if (promise && typeof promise.then == 'function') {
                promise.then(function () {
                  return _this._unrendered(k, oldCont);
                });
              }
            }
            cont = null;
          }
          if (!cont) {
            var c = _this._factory(k, v);
            if (c) {
              onAdd(c, _this._el);
            }
            cont = {
              v: v,
              c: c
            };
            _this._comps[k] = cont;
          }
        };
        for (var k in p) {
          if (_loop(k)) continue;
        }
      }

      // Unrender components
      var _loop2 = function _loop2(_k2) {
        if (p && p.hasOwnProperty(_k2)) {
          return 1; // continue
        }
        var cont = _this._comps[_k2];
        if (cont.c) {
          _this._unrendering[_k2] = cont;
          var promise = onRemove(cont.c, function () {
            return _this._unrenderCont(_k2, cont);
          });
          // Legacy support for promises.
          if (promise && typeof promise.then == 'function') {
            promise.then(function () {
              return _this._unrendered(_k2, cont);
            });
          }
        }
        delete _this._comps[_k2];
      };
      for (var _k2 in this._comps) {
        if (_loop2(_k2)) continue;
      }
    }
  }, {
    key: "_unrenderCont",
    value: function _unrenderCont(k, cont) {
      if (this._unrendering && this._unrendering[k] == cont) {
        if (cont.c) {
          cont.c.unrender();
        }
        delete this._unrendering[k];
      }
    }

    // Legacy support for promises.
  }, {
    key: "_unrendered",
    value: function _unrendered(k, cont) {
      if (this._unrendering && this._unrendering[k] == cont) {
        delete this._unrendering[k];
      }
    }
  }]);
  return ModelFragment;
}();
export default ModelFragment;