function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
/**
 * Update callback function
 * @callback CollectionListener~updateCallback
 * @param {Collection} collection Collection
 * @param {Component} component Component received in CollectionListener constructor.
 * @param {?object} event Collection's event parameters. Null if the callback is triggered by render or setCollection.
 */
/**
 * A helper class for collection components
 */
var CollectionListener = /*#__PURE__*/function () {
  /**
   * Creates a new CollectionListener instance
   * @param {Collection} [collection] Optional collection object
   * @param {Component} component Component
   * @param {CollectionListener~updateCallback} update Callback function called on collection add/remove events and when component is rendered
   * @param {object} [opt] Optional parameters
   * @param {string} [opt.postrenderUpdate] Flag setting if call to update should be done after render. Defaults to false.
   */
  function CollectionListener(collection, component, update, opt) {
    _classCallCheck(this, CollectionListener);
    this.collection = collection;
    this.component = component;
    this.update = update;
    this.rendered = false;
    this._onAdd = this._onEvent.bind(this, 'add');
    this._onRemove = this._onEvent.bind(this, 'remove');
  }
  _createClass(CollectionListener, [{
    key: "onRender",
    value: function onRender() {
      this._setEventListener(true);
      this._update(null);
      this.rendered = true;
    }
  }, {
    key: "onUnrender",
    value: function onUnrender() {
      this._setEventListener(false);
      this.rendered = false;
    }

    /**
     * Set collection
     * If component is rendered, update will be triggered.
     * @param {?Collection} collection Collection
     * @returns {this}
     */
  }, {
    key: "setCollection",
    value: function setCollection(collection) {
      collection = collection || null;
      if (collection === this.collection) {
        return;
      }
      if (this.rendered) {
        this._setEventListener(false);
        this.collection = collection;
        this._setEventListener(true);
        this._update(null);
      } else {
        this.collection = collection;
      }
      return this;
    }

    /**
     * Gets the current collection
     * @returns {?Collection}
     */
  }, {
    key: "getCollection",
    value: function getCollection() {
      return this.collection;
    }
  }, {
    key: "_setEventListener",
    value: function _setEventListener(on) {
      if (!this.collection || !this.collection.on || !this.update) {
        return;
      }
      var cb = on ? 'on' : 'off';
      this.collection[cb]('add', this._onAdd);
      this.collection[cb]('remove', this._onRemove);
    }
  }, {
    key: "_onEvent",
    value: function _onEvent(event, e) {
      e = Object.assign({
        event: event
      }, e);
      this._update(e);
    }
  }, {
    key: "_update",
    value: function _update(e) {
      if (this.update) {
        this.update(this.collection, this.component, e);
      }
    }
  }]);
  return CollectionListener;
}();
export default CollectionListener;