"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _ModelListener = _interopRequireDefault(require("./utils/ModelListener"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
/**
 * Update callback function
 * @callback ModelComponent~updateCallback
 * @param {Model} model Model
 * @param {Component} component Component
 * @param {?object} changed Model's change event parameters. Null if the callback is triggered by render or setModel.
 */
/**
 * A generic component wrapper that listens to change events on a model, calling update on change.
 */
var ModelComponent = /*#__PURE__*/function () {
  /**
   * Creates a new ModelComponent instance
   * @param {Model} [model] Optional model object
   * @param {Component} component Component
   * @param {ModelComponent~updateCallback} update Callback function called on model change and when component is rendered
   * @param {object} [opt] Optional parameters
   * @param {string} [opt.postrenderUpdate] Flag setting if call to update should be done after render. Defaults to false.
   */
  function ModelComponent(model, component, update, opt) {
    _classCallCheck(this, ModelComponent);
    if (typeof component === 'function') {
      update = component;
      component = model;
      model = null;
    }
    this.postrender = !!(opt && opt.postrenderUpdate);
    this.ml = new _ModelListener["default"](model, component, update);
  }

  /**
   * Set model
   * If component is rendered, update will be triggered.
   * @param {?Model} model Model
   * @returns {this}
   */
  _createClass(ModelComponent, [{
    key: "setModel",
    value: function setModel(model) {
      this.ml.setModel(model);
      return this;
    }

    /**
     * Get model.
     * @returns {?Model} Current set model.
     */
  }, {
    key: "getModel",
    value: function getModel() {
      return this.ml.getModel();
    }

    /**
     * Returns the wrapped component
     * @returns {Component} Wrapped component
     */
  }, {
    key: "getComponent",
    value: function getComponent() {
      return this.ml.component;
    }
  }, {
    key: "render",
    value: function render(el) {
      if (!this.postrender) this.ml.onRender();
      var rel = this.ml.component.render(el);
      if (this.postrender) this.ml.onRender();
      return rel;
    }
  }, {
    key: "unrender",
    value: function unrender() {
      this.ml.component.unrender();
      this.ml.onUnrender();
    }
  }]);
  return ModelComponent;
}();
var _default = ModelComponent;
exports["default"] = _default;