"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }
function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor); } }
function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
/**
 * Update callback function
 * @callback ModelListener~updateCallback
 * @param {Model} model Model
 * @param {Component} component Component received in ModelListener constructor.
 * @param {?object} changed Model's change event parameters. Null if the callback is triggered by render or setModel.
 */
/**
 * A helper class for model components
 */
var ModelListener = /*#__PURE__*/function () {
  /**
   * Creates a new ModelListener instance
   * @param {Model} [model] Optional model object
   * @param {Component} component Component
   * @param {ModelListener~updateCallback} update Callback function called on model change and when component is rendered
   */
  function ModelListener(model, component, update) {
    _classCallCheck(this, ModelListener);
    this.model = model;
    this.component = component;
    this.update = update;
    this.rendered = false;
    this._onModelChange = this._onModelChange.bind(this);
  }
  _createClass(ModelListener, [{
    key: "onRender",
    value: function onRender() {
      this._setEventListener(true);
      this._onModelChange(null);
      this.rendered = true;
    }
  }, {
    key: "onUnrender",
    value: function onUnrender() {
      this._setEventListener(false);
      this.rendered = false;
    }

    /**
     * Set model
     * If component is rendered, update will be triggered.
     * @param {?Model} model Model
     * @returns {this}
     */
  }, {
    key: "setModel",
    value: function setModel(model) {
      model = model || null;
      if (model === this.model) {
        return;
      }
      if (this.rendered) {
        this._setEventListener(false);
        this.model = model;
        this._setEventListener(true);
        this._onModelChange(null);
      } else {
        this.model = model;
      }
      return this;
    }

    /**
     * Get model.
     * @returns {?Model} Current set model.
     */
  }, {
    key: "getModel",
    value: function getModel() {
      return this.model;
    }
  }, {
    key: "_setEventListener",
    value: function _setEventListener(on) {
      if (!this.model || !this.model.on || !this.update) {
        return;
      }
      if (on) {
        this.model.on('change', this._onModelChange);
      } else {
        this.model.off('change', this._onModelChange);
      }
    }
  }, {
    key: "_onModelChange",
    value: function _onModelChange(changed) {
      if (this.update) {
        this.update(this.model, this.component, changed);
      }
    }
  }]);
  return ModelListener;
}();
var _default = ModelListener;
exports["default"] = _default;